@@ -133,7 +133,8 @@ class DataPlate:
               request_timeout=None,
               es_index_type=None,
               bucket_suffixes=None,
-              bucket_filter=None):
+              bucket_filter=None,
+              force_scheme_change=False):
         """
         Executes remote SQL query, and saves results to the specified file.
 
@@ -156,6 +157,10 @@ class DataPlate:
             bucket option - bucket path suffix added to your dataset path name, [e.g.: MyPathSuffix1,MyPathSuffix2] (default: None)
         bucket_filter: str
             bucket option - include files in the bucket with file names matching the pattern (default: None)
+        force_scheme_change: boolean
+            In case scheme was change in the data meant to read/query use this to indicate spark to re-create the temporary view
+            Use this only if case you want to read parquet files that were written with dataset=True and mode="overwrite_partitions"
+            and where the overwrite scheme was changed
         """
         headers = {'X-Access-Key': self.access_key}
         params = {}
@@ -170,6 +175,8 @@ class DataPlate:
             params['bucket_suffixes'] = bucket_suffixes
         if bucket_filter:
             params['bucket_filter'] = bucket_filter
+        if force_scheme_change:
+            params['force_scheme_change'] = force_scheme_change
 
         retries = 1
         while True:
@@ -216,6 +223,7 @@ class DataPlate:
                     es_index_type=None,
                     bucket_suffixes=None,
                     bucket_filter=None,
+                    force_scheme_change=False,
                     **kwargs):
         """
         Executes remote SQL query, and returns Pandas dataframe.
@@ -238,6 +246,10 @@ class DataPlate:
             bucket option - bucket path suffix added to your dataset path name, [e.g.: MyPathSuffix1,MyPathSuffix2] (default: None)
         bucket_filter: str
             bucket option - include files in the bucket with file names matching the pattern (default: None)
+        force_scheme_change: boolean
+            In case scheme was change in the data meant to read/query use this to indicate spark to re-create the temporary view
+            Use this only if case you want to read parquet files that were written with dataset=True and mode="overwrite_partitions"
+            and where the overwrite scheme was changed
         **kwargs : params
             Arbitrary parameters to pass to `pandas.read_json()` method
 
@@ -248,7 +260,7 @@ class DataPlate:
         import pandas as pd
         with tempfile.NamedTemporaryFile(suffix='.gz') as t:
             output_file = t.name
-            self.query(query, output_file, refresh, async_m, request_timeout, es_index_type, bucket_suffixes, bucket_filter)
+            self.query(query, output_file, refresh, async_m, request_timeout, es_index_type, bucket_suffixes, bucket_filter, force_scheme_change)
             with open(output_file, 'rb') as fh:
                 return pd.read_json(fh, compression='gzip', lines=True, **kwargs)
 
@@ -615,7 +627,7 @@ class DataPlate:
 
 
     def run_notebook(self, notebook_file_path, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
-        """Run a notebook in SageMaker Processing producing a new output notebook.
+        """ Run a notebook in SageMaker Processing producing a new output notebook.
         Args:
             notebook (str): The notebook file path.
             input_path (str): The S3 object containing the notebook. If this is None, the `notebook` argument is
@@ -651,7 +663,7 @@ class DataPlate:
                 params['parameters'] = parameters
             if max_time_limit_minutes:
                 params['timelimit_minutes'] = max_time_limit_minutes
-            params['SecurityGroupIds'] =  json.dumps(securityGroupIds)#','.join(['"%s"' % w for w in securityGroupIds])
+            params['SecurityGroupIds'] = json.dumps(securityGroupIds)#','.join(['"%s"' % w for w in securityGroupIds])
             params['Subnets'] = json.dumps(subnets)#','.join(['"%s"' % w for w in subnets])
             params['notebook_name'] = notebook_name if notebook_name else ""
             if role:
@@ -1186,16 +1198,16 @@ class DataPlate:
 
 
     def schedule_notebook(self, notebook_file_path, rule_name = None, schedule = None, event_pattern = None, instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
-        """Create a schedule for running a notebook in SageMaker Processing.
+        """ Create a schedule for invoking a notebook in a specific cron/rate based intervals
 
-            Creates a CloudWatch Event rule to invoke the installed Lambda either on the provided schedule or in response
-            to the provided event. \
+            Creates a scheduled rule to invoke the notebook (calling run_notebook) on the provided schedule or in response
+            to the provided event \
 
-            :meth:`schedule` can upload a local notebook file to run or use one previously uploaded to S3. 
+            :meth:
             To find jobs run by the schedule, see :meth:`list_runs_notebook` using the `rule` argument to filter to 
             a specific rule. To download the results, see :meth:`download_notebook_result` 
 
-            job = run.schedule_notebook(notebook="powers.ipynb", rule_name="Powers", schedule="rate(1 hour)")
+            dataplate.schedule_notebook(notebook="powers.ipynb", rule_name="Powers", schedule="rate(1 hour)")
 
             Args:
                 notebook (str): The notebook file path.
@@ -1237,8 +1249,8 @@ class DataPlate:
                 params['parameters'] = parameters
             if max_time_limit_minutes:
                 params['timelimit_minutes'] = max_time_limit_minutes
-            params['SecurityGroupIds'] = ",".join(securityGroupIds)
-            params['Subnets'] = ",".join(subnets)
+            params['SecurityGroupIds'] = json.dumps(securityGroupIds)
+            params['Subnets'] = json.dumps(subnets)
             params['notebook_name'] = notebook_name if notebook_name else ""
             if not rule_name:
                 raise Error(f'rule_name is required')
@@ -1298,6 +1310,217 @@ class DataPlate:
             raise Error(e)
 
 
+    def run_notebook_after_schedule(self, notebook_file_path, rule_name = None, trigger_rule_name = None, trigger_rule_status = "Completed" ,instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
+        """invoke a notebook run after another scheduled notebook Completed/InProgress/Failed.
+
+            Creates a rule to invoke the notebook once a scheduled notebook rule status changed to Completed/InProgress/Failed
+
+            :meth:
+            To find jobs run by the schedule, see :meth:`list_runs_notebook` using the `rule` argument to filter to
+            a specific rule. To download the results, see :meth:`download_notebook_result`
+
+            dataplate.run_notebook_after_schedule(notebook="powers.ipynb", rule_name="Powers-follow", trigger_rule_name="Powers")
+
+            Args:
+                notebook (str): The notebook file path.
+                rule_name (str): The name of the rule for CloudWatch Events (required).
+                trigger_rule_name (str): A scheduled rule name of another scheduled notebook that will trigger this notebook once Completed/InProgress/Failed
+                trigger_rule_status (str): The state of the trigger_rule_name scheduled notebook that this notebook will listen for invokation (default='Completed')
+                                           options: 'Completed','InProgress','Failed'
+                input_path (str): The S3 object containing the notebook. If this is None, the `notebook` argument is
+                                  taken as a local file to upload (default: None).
+                parameters (dict): The dictionary of parameters to pass to the notebook (default: {}).
+                instance_type (str): The SageMaker instance to use for executing the job (default: ml.m5.large).
+                max_time_limit_minutes : maximum minutes to run before force stop
+                securityGroupIds : a list of securityGroup Ids of aws for the processing job to communicate with, in case communication with other resources, e.g. internal dataplate service, is needed
+                subnets : a list of subnets of aws for the processing job to communicate with
+                role : a role ARN to run the notebook, the default is the Dataplate service role (cross rols in case of Sass)
+            """
+        try:
+            if not notebook_file_path or not os.path.isfile(notebook_file_path) or not notebook_file_path.endswith('.ipynb'):
+                raise FileNotFoundError(f'notebook file is not legal/valid : {notebook_file_path if notebook_file_path else "None"}')
+
+            f = open(notebook_file_path, 'r')
+            if f:
+                # Reading from file
+                notebook_json_data = json.loads(f.read())
+                # Closing file
+                f.close()
+
+            if not notebook_json_data or len(json.dumps(notebook_json_data)) < 10 or not 'cells' in notebook_json_data:
+                raise Error(f'notebook file is not legal : {notebook_file_path if notebook_file_path else "None"}')
+
+            notebook_name = os.path.basename(notebook_file_path)
+            params = {}
+            if instance_type:
+                params['instance_type'] = instance_type
+            if parameters:
+                params['parameters'] = parameters
+            if max_time_limit_minutes:
+                params['timelimit_minutes'] = max_time_limit_minutes
+            params['SecurityGroupIds'] = json.dumps(securityGroupIds)
+            params['Subnets'] = json.dumps(subnets)
+            params['notebook_name'] = notebook_name if notebook_name else ""
+            if not rule_name:
+                raise Error(f'rule_name is required')
+            if not trigger_rule_name:
+                raise Error(f'trigger_rule_name is required, for following a specific scheduled rule that will trigger this notebook')
+            if trigger_rule_status:
+                if trigger_rule_status != 'Completed' and trigger_rule_status != 'InProgress' and trigger_rule_status != 'Failed':
+                    raise Error(
+                        f"trigger_rule_status is invalid, options: 'Completed','InProgress','Failed'")
+            params['rule_name'] = rule_name
+            params['trigger_rule_name'] = trigger_rule_name
+            params['trigger_rule_status'] = trigger_rule_status
+            if role:
+                params['role'] = role
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('Uploading data...')
+                r = self.session.post( \
+                    '{}/api/aws/runNotebookAfterSchedule'.format(self.base_url), params=params, data=json.dumps(notebook_json_data),
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('creating rule to invoke notebook after scheduled notebook, processing...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    rJson = json.loads(r.text)
+                    logging.info('rule to invoke notebook finished successfully.')
+                    return json.dumps(rJson)
+                else:
+                    logging.exception('Could not find proper output, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('invoke_notebook_after_schedule , ' + str(e))
+            raise Error(e)
+
+
+
+    def run_notebook_after_notebook(self, notebook_file_path, rule_name = None, trigger_notebook_name = None, trigger_notebook_status = "Completed" ,instance_type = "ml.m5.large", parameters = "{}", max_time_limit_minutes = 180, securityGroupIds = [], subnets= [], role = None):
+        """invoke a notebook run after another scheduled notebook Completed/InProgress/Failed.
+
+            Creates a rule to invoke the notebook once a scheduled notebook rule status changed to Completed/InProgress/Failed
+
+            :meth:
+            To find jobs run by the schedule, see :meth:`list_runs_notebook` using the `rule` argument to filter to
+            a specific rule. To download the results, see :meth:`download_notebook_result`
+
+            dataplate.run_notebook_after_schedule(notebook="powers.ipynb", rule_name="myNotebook-power-follow", trigger_notebook_name="myNotebook.ipynb")
+
+            Args:
+                notebook (str): The notebook file path.
+                rule_name (str): The name of the rule for CloudWatch Events (required).
+                trigger_notebook_name (str): A notebook name of another notebook that will trigger this notebook once Completed/InProgress/Failed
+                trigger_notebook_status (str): The state of the trigger_notebook_name notebook that this notebook will listen for invokation (default='Completed')
+                                           options: 'Completed','InProgress','Failed'
+                input_path (str): The S3 object containing the notebook. If this is None, the `notebook` argument is
+                                  taken as a local file to upload (default: None).
+                parameters (dict): The dictionary of parameters to pass to the notebook (default: {}).
+                instance_type (str): The SageMaker instance to use for executing the job (default: ml.m5.large).
+                max_time_limit_minutes : maximum minutes to run before force stop
+                securityGroupIds : a list of securityGroup Ids of aws for the processing job to communicate with, in case communication with other resources, e.g. internal dataplate service, is needed
+                subnets : a list of subnets of aws for the processing job to communicate with
+                role : a role ARN to run the notebook, the default is the Dataplate service role (cross rols in case of Sass)
+            """
+        try:
+            if not notebook_file_path or not os.path.isfile(notebook_file_path) or not notebook_file_path.endswith('.ipynb'):
+                raise FileNotFoundError(f'notebook file is not legal/valid : {notebook_file_path if notebook_file_path else "None"}')
+
+            f = open(notebook_file_path, 'r')
+            if f:
+                # Reading from file
+                notebook_json_data = json.loads(f.read())
+                # Closing file
+                f.close()
+
+            if not notebook_json_data or len(json.dumps(notebook_json_data)) < 10 or not 'cells' in notebook_json_data:
+                raise Error(f'notebook file is not legal : {notebook_file_path if notebook_file_path else "None"}')
+
+            notebook_name = os.path.basename(notebook_file_path)
+            params = {}
+            if instance_type:
+                params['instance_type'] = instance_type
+            if parameters:
+                params['parameters'] = parameters
+            if max_time_limit_minutes:
+                params['timelimit_minutes'] = max_time_limit_minutes
+            params['SecurityGroupIds'] = json.dumps(securityGroupIds)
+            params['Subnets'] = json.dumps(subnets)
+            params['notebook_name'] = notebook_name if notebook_name else ""
+            if not rule_name:
+                raise Error(f'rule_name is required')
+            if not trigger_notebook_name:
+                raise Error(f'trigger_notebook_name is required, for following a specific notebook rule that will trigger this notebook')
+            if trigger_notebook_status:
+                if trigger_notebook_status != 'Completed' and trigger_notebook_status != 'InProgress' and trigger_notebook_status != 'Failed':
+                    raise Error(
+                        f"trigger_rule_status is invalid, options: 'Completed','InProgress','Failed'")
+            params['rule_name'] = rule_name
+            params['trigger_rule_name'] = trigger_notebook_name
+            params['trigger_rule_status'] = trigger_notebook_status
+            if role:
+                params['role'] = role
+
+            headers = {'X-Access-Key': self.access_key, 'Content-Type': 'application/json'}
+
+            try:
+                logging.info('Uploading data...')
+                r = self.session.post( \
+                    '{}/api/aws/runNotebookAfterSchedule'.format(self.base_url), params=params, data=json.dumps(notebook_json_data),
+                    headers=headers, stream=True, allow_redirects=False)
+
+                if r.status_code != 200:
+                    if r.status_code == 302:
+                        raise Error(
+                            'Bad Access Key! Get your access key at: {}'.format(
+                                self.base_url))
+                    if r.status_code == 206:
+                        logging.info('creating rule to invoke notebook after scheduled notebook, processing...')
+                        time.sleep(5)
+                    raise Error(
+                        'Bad HTTP exit status returned from the API: {}. Error was: {}'.
+                            format(r.status_code, r.text))
+
+                logging.info('Parsing response')
+                # logging.info(str(r.text))
+                if r.text:
+                    rJson = json.loads(r.text)
+                    logging.info('rule to invoke notebook finished successfully.')
+                    return json.dumps(rJson)
+                else:
+                    logging.exception('Could not find proper output, please check your parameters')
+                # return r.text
+
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.ReadTimeout) as e:
+                logging.exception('Got ConnectionError/ReadTimeout exception.')
+                raise Error(e)
+
+        except Exception as e:
+            logging.exception('invoke_notebook_after_notebook , ' + str(e))
+            raise Error(e)
+
+
     def scan_notebook(self, notebook_file_path, parameters = "{}"):
         """Scan a notebook Job result for security issue.
         Args:
